# Function to calculate Muller's Ratchet
#
# Parameters
#   u:         Poisson parameter for new mutations
#   s:         selection coefficient
#   init.freq: initial frequencies of the "mutation classes",
#              - frequencies don't need to be normalized
#              - you can choose as many classes as you want to
#   stoptime:   number of generations that shall be calculated
#   n.class:    number of classes computed
#   Returns
#   list for frequencies for all stoptime generations

mullersRatchet <- function( u=0.1, s=0.05, init.freq=c(1),
                            stoptime=50, n.class=20 ){

   # number of classes
   if(length(init.freq)<n.class) {
     init.freq<-c(init.freq, 0* (length(init.freq)+1):n.class)
   } else {
     init.freq<-init.freq[1:n.class]
   }

   # normalize frequencies
   freq<-matrix(0, nrow=stoptime, ncol=n.class)
   freq[1,]<-init.freq / sum( init.freq )

   # calculate over stoptime generations
   for ( gen in 2:stoptime ) {
      # empty vector of frequencies after mutation
      freq.after.mut <- rep( 0, n.class )

      # calculate frequencies after mutation for each class

      for ( i in 1:n.class ) {
        freq[gen,i]<-sum(freq[gen-1, i:1]*(1-s)^((i-1):0) * dpois( 0:(i-1), u ))
      }
      freq[gen,]<-freq[gen,]/(sum(freq[gen,]))
   }

   res <- list( u=u, s=s, stoptime=stoptime, freq=freq )
   class( res ) <- "mullersRatchet"

   return( res ) 
}



plot.mullersRatchet <- function( x, gen=x$stoptime, what=c( "freq", "dpois" ), col=vector(),
                                 xlim=NULL, ylim=NULL,
                                 xlab="no. of mutations", ylab="frequency", 
                                 legend.pos="topright", ... )
{
   n.class <- length( x$freq[1,] )

   legend <- vector()
   legend.col <- vector()

   col["freq"]  = ifelse( is.na( col["freq"] ), "red", col["freq"] )
   col["dpois"] = ifelse( is.na( col["dpois"] ), "blue", col["dpois"] )

   if ( is.null( xlim ) ) {
      xlim <- c( 0, n.class - 1 )
   }

   if ( is.null( ylim ) ) {
      y.values <- vector()

      if ( "freq" %in% what ) {
         y.values <- c( y.values, x$freq[gen,] )
      }
      if  ( "dpois" %in% what ) {
         y.values <- c( y.values, dpois( 0:(n.class-1 ), x$u/x$s ) )
      }
      
      ylim <- c( 0, max( y.values ) )
   }
   
   plot( 0, 0, xlim=xlim, ylim=ylim, type="n", xlab=xlab, ylab=ylab, ... )

   if ( "freq" %in% what ) {
      par( new=TRUE )
      plot( 0:(n.class-1), x$freq[gen,], type="h", ylim=ylim, xlim=xlim,
            xlab="", ylab="", col=col["freq"] )
      legend <- c( legend, "freq" )
      legend.col <- c( legend.col, col["freq"] )
   }

   if ( "dpois" %in% what ) {
      points( 0:(n.class-1), dpois( 0:(n.class-1 ), x$u/x$s ),
              col=col["dpois"] )
      legend <- c( legend, "dpois" )
      legend.col <- c( legend.col, col["dpois"] )
   }
     
   legend( legend.pos, legend=legend,
           col=legend.col, lty=1, bg="white" )
}



